<?php
/**
 * This file contains the installer for iOS devices and Apple 10.7 Lion
 *
 *
 * @author Stefan Winter <stefan.winter@restena.lu>
 * @package Developer
 */

/**
 * 
 */
require_once('Device.php');

// set_locale("devices");

/**
 * This is the main implementation class of the module
 *
 * The class should only define one public method: writeInstaller.
 *
 * All other methods and properties should be private. This example sets zipInstaller method to protected, so that it can be seen in the documentation.
 *
 * @package Developer
 */
class Device_welcomeletter extends DeviceConfig {
    final public function __construct($device) {
      debug(4,"got device: $device\n");
      $this->supportedEapMethods  = array(EAP::$TLS, EAP::$PEAP_MSCHAP2, EAP::$TTLS_PAP, EAP::$TTLS_GTC, EAP::$FAST_GTC, EAP::$TTLS_MSCHAP2);
      debug(4,"This device supports the following EAP methods: ");
      debug(4,$this->supportedEapMethods);
    }


  /**
   * this array holds the list of EAP methods supported by this device
   */
   // public static $my_eap_methods = array(EAP::$TLS, EAP::$PEAP_MSCHAP2, EAP::$TTLS_PAP, EAP::$FAST_GTC);

//   public static $my_eap_methods = array(array("OUTER" => TLS, "INNER" => NONE), array("OUTER" => PEAP, "INNER" => MSCHAPv2), array("OUTER" => TTLS, "INNER" => NONE));

   /**
   * prepare a zip archive containing files and settings which normally would be used inside the module to produce an installer
   *
   * {@source}
   */
   public function writeInstaller(Profile $profile) {
   /** run innitial setup
      this will:
       - create the temporary directory and save its path as $this->FPATH
       - process the CA certificates and store results in $this->attributes['internal:CAs'][0]
            $this->attributes['internal:CAs'][0] is an array of processed CA certificates
            a processed certifincate is an array 
               'pem' points to pem feromat certificate
               'der' points to der format certificate
               'md5' points to md5 fingerprint
               'sha1' points to sha1 fingerprint
               'name' points to the certificate subject
       - save the info_file (if exists) and put the name in $this->attributes['internal:info_file_name'][0]
   */
//    $this->supportedEapMethods = Device_welcomeletter::$my_eap_methods;

    debug(4,"mobileconfig Module Installer start\n");
//    $this->setup($profile);

    $this->copyFile('welcomeletter.odt');

    $o = system('unzip -q -d ./ welcomeletter.odt');

    $filename = 'content.xml';

    

    $source = fopen($filename,'r+');
    $content = fread($source, 1000000);
    ftruncate($source, 0);
    $content = preg_replace('/XXXINSTITUTIONXXX/',$this->attributes['internal:inst_name'][0],$content);
    $content = preg_replace('/XXXPROFILEXXX/',$this->attributes['internal:profile_name'][0],$content);
    $helpdesktext = "";

    if ($this->attributes['support:email'])
            foreach ($this->attributes['support:email'] as $number => $option)
                $helpdesktext .= "E-Mail: <text:span text:style-name=\"T2\">".$option."</text:span></text:p><text:p style-name=\"Standard\">";
    if ($this->attributes['support:eap_types'])
            foreach ($this->attributes['support:eap_types'] as $number => $option)
                $helpdesktext .= "Special EAP Type support URL: <text:span text:style-name=\"T1\">".$option."</text:span></text:p><text:p style-name=\"Standard\">";
    if ($this->attributes['support:phone'])
            foreach ($this->attributes['support:phone'] as $number => $option)
                $helpdesktext .= "Phone: <text:span text:style-name=\"T2\">".$option."</text:span></text:p><text:p style-name=\"Standard\">";
    if ($this->attributes['support:url'])
            foreach ($this->attributes['support:url'] as $number => $option)
                $helpdesktext .= "Web: <text:span text:style-name=\"T2\">".$option."</text:span></text:p><text:p style-name=\"Standard\">";
    
    $content = preg_replace('/XXXHELPDESKXXX/',$helpdesktext,$content);
    fseek($source,0, SEEK_SET);
    fwrite($source,$content);
    fclose($source);
    $o = system('zip -q welcomeletter.odt *');
    // openoffice needs to be running on headless TCP/8100 mode on the system...
    // python needs to be python2.7
    //$o2 = system('python /root/DocumentConverter.py '.$this->FPATH.'/welcomeletter.odt '.$this->FPATH.'/welcomeletter.pdf');
    return 'welcomeletter.odt';
   }

}
