<?php
require_once(dirname(dirname(dirname(__FILE__)))."/config/_config.php");
include_once("IdP.php");
include_once("Profile.php");
include_once("Helper.php");
include_once("inc/common.inc.php");

if (isset($_GET['lang'])) {
    $ourlocale = set_locale("web_admin",$_GET['lang']);
    $lang_string = "&lang=".$_GET['lang'];
}
else {
    $ourlocale = set_locale("web_admin");
    $lang_string="";
}



function naptr($realm) {
    $NAPTRs = dns_get_record($realm.".", DNS_NAPTR);
    if ($NAPTRs !== FALSE && count($NAPTRs) > 0) {
        echo "<p>".sprintf(_("This realm has %d NAPTR records, "),count($NAPTRs));
        $NAPTRs_eduroam = array();
        foreach ($NAPTRs as $naptr) {
            if ($naptr["services"] == "x-eduroam:radius.tls" )
                $NAPTRs_eduroam[] = $naptr;
        }

        if (count($NAPTRs_eduroam) > 0) {
            echo sprintf(_("out of which %d are eduroam NAPTR records."), count($NAPTRs_eduroam))."</p>";
            return $NAPTRs_eduroam;
        } else {
            echo _("but none are eduroam NAPTR records.")."</p>";
            return FALSE;
        }
    }
    else {
        echo "<p>"._("There are no NAPTR records for this realm.")."<p>";
        return FALSE;
    }

}

function format_compliance($NAPTRs, &$errors ) {
    $format_errors = array();
    echo "<p>"._("Checking NAPTR format compliance: flag = S and regex = (empty) ...")."</p>";
    foreach ($NAPTRs as $edupointer) {
// must be "s" type for SRV
        if ( $edupointer["flags"] != "s" ) {
            echo "<p>"._("<strong>Error:</strong> eduroam discovery profile only supports 'S' type NAPTRs.")."</p>";
            $format_errors[] = "FLAG";
        }
// no regex
        if ( $edupointer["regex"] != "" ) {
            echo "<p>"._("<strong>Error:</strong> eduroam discovery profile only supports NAPTRs with empty regex.")."</p>";
            $format_errors[] = "REGEX";
        }
    }
    if (count($format_errors) > 0) {
        $errors = array_merge($errors,$format_errors);
        return FALSE;
    } else
        return TRUE;
}

function srv_resolution($NAPTRs,&$errors) {
    echo "<p>"._("Trying to resolve the SRVs into host names ... ")."</p>";
    $SRV_errors = array();
    $SRV_targets = array();

    foreach ( $NAPTRs as $edupointer) {
        $temp_result = dns_get_record($edupointer["replacement"], DNS_SRV);
        if ($temp_result === FALSE || count($temp_result) == 0) {
            echo "<p>".sprintf(_("<strong>Error:</strong> SRV entry %s could not be resolved!"),$edupointer["replacement"])."</p>";
            $SRV_errors[] = "SRV_NO_TARGET";
        } else
            foreach ($temp_result as $res)
                $SRV_targets[] = array ("hostname" => $res["target"], "port" => $res["port"]);
    }
    echo "<p>".sprintf(_("%d host names discovered."),count($SRV_targets))."</p>";
    $errors = array_merge($errors,$SRV_errors);
    return $SRV_targets;
}

function name_resolution($hosts,&$errors) {
    $ip_addresses = array();
    $resolution_errors = array();

    foreach ($hosts as $server) {
        $host_resolution_6 = dns_get_record($server["hostname"],DNS_AAAA);
        $host_resolution_4 = dns_get_record($server["hostname"],DNS_A);
        $host_resolution = array_merge($host_resolution_6, $host_resolution_4);
        if ($host_resolution === FALSE || count($host_resolution) == 0) {
            echo "<p>".sprintf(_("<strong>Error:</strong> Host name %s could not be resolved!"),$server["hostname"])."</p>";
            $resolution_errors[] = "HOST_NO_ADDRESS";
        } else
            foreach ($host_resolution as $address)
                if (isset($address["ip"]))
                    $ip_addresses[] = array("family" => "IPv4", "IP" => $address["ip"], "port" => $server["port"]);
                else
                    $ip_addresses[] = array("family" => "IPv6", "IP" => $address["ipv6"], "port" => $server["port"]);
    }

    echo "<p>".sprintf(_("%d IP addresses discovered."),count($ip_addresses))."</p>";
    $errors = array_merge($errors,$resolution_errors);
    return $ip_addresses;
}
header("Content-Type:text/html;charset=utf-8");
?>
<!DOCTYPE html>
<html>
    <head>
        <meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
        <meta http-equiv="content-language" content="<?php echo $ourlocale;?>">
        <title><?php echo _("Sanity check for dynamic discovery of realms");?></title>
        <link rel="stylesheet" type="text/css" href="../resources/css/cat.css" />
    </head>
    <body>
        <?php
        include "inc/admin_header.php";
        // let's check if the inst and profile handles actually exists in the DB

        $my_inst = new IdP($_GET['inst_id']);

        if (! $my_inst instanceof IdP) {
            echo "<p>"._("The page was called with a non-existing IdP identifier. This is not supposed to happen!")."</p>";
            exit(1);
        }

        $my_profile = new Profile($_GET['profile_id']);

        if (! $my_profile instanceof Profile) {
            echo "<p>"._("The page was called with a non-existing profile identifier. This is not supposed to happen!")."</p>";
            exit(1);
        }

        if ($my_profile->realm != "") { // doing the checks
            $failimage = "<img class='icon' src='../resources/images/icons/Exclamation-orange-icon.png' alt='"._("Error!")."' title='"._("Error!")."'/>";
            $goodimage = "<img class='icon' src='../resources/images/icons/Checkmark-lg-icon.png' alt='"._("OK!")."' title='"._("OK!")."'/>";
            $translate = _("STATIC");
            $translate = _("DYNAMIC");
            $errorstate = array();
            $result = "STATIC";

            echo "<h1>".sprintf(_("Checking realm %s"),$my_profile->realm)."</h1>";
            echo "<form action='none'>
                <fieldset class='option_container'><legend><strong>"._("DNS checks")."</strong></legend>";

            if ($eduroam_NAPTRs = naptr($my_profile->realm)) {
                $result = "DYNAMIC";
                format_compliance($eduroam_NAPTRs,$errorstate);
                if (count($errorstate) > 0)
                    echo "<p>"._("Skipping further discovery checks because of previous errors!")."</p>";
                else {
                    $hostnames = srv_resolution($eduroam_NAPTRs,$errorstate);
                    $ip_addresses = name_resolution($hostnames,$errorstate);
                }

            }

            echo "<p><table><tr><td>".(count($errorstate) == 0 ? $goodimage : $failimage )."</td><td>".sprintf(_("Realm is <strong>%s</strong> "),_($result));
            if (count($errorstate) == 0) {
                echo _("with no DNS errors encountered. Congratulations!")."</td></tr></table></p>";
            } else {
                echo _("but there were DNS errors! Check them!")."</td></tr></table></p><ul>";
                foreach ($errorstate as $token)
                    echo "<li>".$token."</li>";
                echo "</ul>";
            }

            echo "</fieldset>";
            // always do the static reachability checks
            echo "<fieldset class='option_container'>
                       <legend><strong>"._("STATIC connectivity tests")."</strong></legend>
                       <p>"._("The check sends a request for your realm through various entry points of the eduroam infrastructure. The request will contain the 'Operator-Name' attribute, and will be larger than 1500 Bytes to catch two common configuration problems.<br/>Since we don't have an account at your realm, we can't authenticate successfully - so the expected outcome is to get an Access-Reject after having gone through an EAP conversation.");
            foreach (Config::$RADIUS_HOSTS as $host) {
                echo "<p>".sprintf(_("Checking from <strong>%s</strong>: "),$host['display_name'])."</p>";
                flush();
                $packetflow = array();
                $anon_id = "";
                if ($my_profile->use_anon_outer) $anon_id = "-A @".$my_profile->realm;
                $cmdline = "rad_eap_test -c -H ".$host['ip']." -P 1812 -S ".$host['secret']." -M 22:44:66:CA:20:00 $anon_id -u eduroam-cat-connectivity-test@".$my_profile->realm." -p nopassword -e TTLS -m WPA-EAP -t ".$host['timeout']." | grep 'RADIUS message:' | cut -d ' ' -f 3 | cut -d '=' -f 2";
                exec($cmdline,$packetflow);
                $packetcount = array_count_values($packetflow);
                
                // check if RADIUS request gets rejected timely
                // this means we expect to see Requests, Challenges and Accepts, and the numbers should add up
                // the lack of a challenge means premature rejection - not good
                if (! isset($packetcount[3]) ) { // no final reject. hm.
                    if (isset($packetcount[11])) // but there was an Access-Challenge


                        echo "<p><table><tr><td>$failimage"."</td><td>".sprintf(_("<strong>Test FAILED</strong>: there was a bidirectional RADIUS conversation, but it did not finish after %d seconds!"),$host['timeout'])."</td></tr></table></p>";
                    else
                        echo "<p><table><tr><td>$failimage"."</td><td>".sprintf(_("<strong>Test FAILED</strong>: no reply from the RADIUS server after %s seconds. Either the responsible server is down, or routing is broken!"),$host['timeout'])."</td></tr></table></p>";
                }
                elseif ( isset($packetcount[3]) && ! isset($packetcount[11])) {
                    echo "<p><p><table><tr><td>$failimage"."</td><td>"._("<strong>Test FAILED</strong>: the request was rejected immediately, without EAP conversation! Either the responsible server is down, or routing is broken!")."</td></tr></table></p>";
                }
                elseif (isset($packetcount[3]) && isset($packetcount[11]) && $packetcount[1] == $packetcount[11]+$packetcount[3] ) {
                    echo "<p><p><table><tr><td>$goodimage"."</td><td>"._("<strong>Test successful</strong>: a bidirectional RADIUS conversation with multiple round-trips was carried out, and ended in an Access-Reject as planned.")."</td></tr></table></p>";
                }
                else
                    echo "<p><p><table><tr><td>$failimage"."</td><td>".sprintf(_("<strong>Strange</strong>: there is a packet count mismatch. Maybe bad connectivity and retransmits? I had %d Access-Requests, %d Access-Challenges and %d Access-Rejects."),$packetcount[1],$packetcount[11],$packetcount[3])."</td></tr></table></p>";
            }
            echo "</fieldset>";


            if ($result == "DYNAMIC") {
                echo "<fieldset class='option_container'>
                           <legend><strong>"._("DYNAMIC connectivity tests")."</strong></legend>";

                if (count($ip_addresses) > 0) {
                    foreach ($ip_addresses as $addr) {
                        if ($addr['family'] == "IPv6") {
                            echo "<strong>".$addr['IP']. " TCP/".$addr['port']."</strong><ul style='list-style-type: none;'><li>"._("Due to OpenSSL limitations, it is not possible to check IPv6 addresses at this time.")."</li></ul>";
                            continue;
                        }

                        $bracketaddr=($addr["family"] == "IPv6" ? "[".$addr["IP"]."]" : $addr["IP"]);
                        echo "<p><strong>".$bracketaddr." TCP/".$addr['port']."</strong><ul style='list-style-type: none;'>";
                        $opensslbabble = array();
                        exec("openssl s_client -connect ".$bracketaddr.":".$addr['port']." -tls1 -CApath ".Config::$BASE_DIR."/config/ca-certs/ 2>&1",$opensslbabble);
                        // echo "Result: ".print_r($opensslbabble);
                        if (preg_match('/verify error:num=19/', implode($opensslbabble)))
                            echo "<li><table><tr><td>$failimage</td><td>"._("<strong>ERROR</strong>: the server presented a cerificate which is from an unknown authority!")."</td></tr></table></li>";
                        if (preg_match('/verify return:1/', implode($opensslbabble)))
                            echo "<li><table><tr><td>$goodimage</td><td>"._("Server handshake completed.")."</td></tr></table></li>";
                        echo "</ul>";
                    }
                    echo "</fieldset>";

                }



                // further checks TBD:
                //     check if accepts certificates from all accredited CAs
                //     check if doesn't accept revoked certificates
                //     check if RADIUS request gets rejected timely
                //     check if truncates/dies on Operator-Name


            }
            echo "</form>";

        } else {
            echo "<p>$failimage"._("Your profile does not contain a valid realm name. We can only check the validity of a realm if you let us know the realm name. Please consider going to your profile properties and enter the realm name.")."</p>";
        }
        echo "<form method='post' action='dashboard.php?inst_id=$my_inst->identifier$lang_string'><button type='submit' name='submitbutton' value='return'>"._("Return to dashboard")."</button></form>";
        ?>

    </body>
</html>
