<?php

require_once(dirname(dirname(dirname(dirname(__FILE__))))."/config/_config.php");
include_once("Helper.php");
include_once("X509.php");
include_once("EAP.php");

function display_name($input) {
    $DisplayNames = array ( _("Support: Web") => "support:url",
            _("Support: EAP Types") => "support:eap_types",
            _("Support: Phone") => "support:phone",
            _("Support: E-Mail") => "support:email",
            _("Location") => "general:geo_coordinates",
            _("Logo URL") => "general:logo_url",
            _("Logo image") => "general:logo_file",
            _("Additional SSID") => "general:SSID",
            _("Additional SSID (with WPA/TKIP)") => "general:SSID_with_legacy",
            _("Name of Authentication Server") => "eap:server_name",
            _("Terms of Use") => "general:info_file",
            _("CA Certificate URL") => "eap:ca_url",
            _("CA Certificate File") => "eap:ca_file",
            _("Institution Name") => "inst_name",
            _("PEAP-MSCHAPv2") => EAP::$PEAP_MSCHAP2,
            _("TLS") => EAP::$TLS,
            _("TTLS-PAP") => EAP::$TTLS_PAP,
            _("TTLS-MSCHAPv2") => EAP::$TTLS_MSCHAP2,
            _("TTLS-GTC") => EAP::$TTLS_GTC,
            _("FAST-GTC") => EAP::$FAST_GTC,
    );
    $find = array_search($input,$DisplayNames);

    if ( $find === FALSE ) {
        return $input;
    }
    else {
        return $find;
    }
}

function check_upload_sanity($optiontype, $filename) {

    // we check logo_file with ImageMagick

    if ($optiontype == "general:logo_file") {
        $image = new Imagick();
        try {
            $image->readImage($filename);
        } catch (ImagickException $e) {
            echo "Error".$e->getMessage();
            return FALSE;
        }
        // echo "Image survived the sanity check.";
        return TRUE;
    };

    // imported logos from URL are present as binary string, not filename

    if ($optiontype == "internal:logo_from_url") {
        $image = new Imagick();
        try {
            $image->readImageBlob($filename);
        } catch (ImagickException $e) {
            echo "Error".$e->getMessage();
            return FALSE;
        }
        // echo "Image survived the sanity check.";
        return TRUE;
    };
    
    // we check CA files with X.509 routines

    if ($optiontype == "eap:ca_file") {
        echo "Checking $optiontype with file $filename";
        if( $cert = X509::downloadCA("file:///".$filename) )
            return TRUE;
        // echo "Error! The certificate seems broken!";
        return FALSE;
    }

    // ToU files are checked by guessing the mime type of the file content
    // some mime types are white-listed, the rest is rejected

    if ($optiontype == "general:info_file") {
        $info = new finfo();
        $filetype = $info->file($filename, FILEINFO_MIME_TYPE);

        if ($filetype == "application/rtf"
                || $filetype == "text/rtf"
                || $filetype == "text/plain"
        // || $filetype == "application/rtf"

        )
            return TRUE;
    }

    return FALSE;
}

function processSubmittedFields($object,$pendingattributes) {

    // construct new array with all non-empty options for later feeding into DB

    $options = array();
    $killlist = $pendingattributes;

    $a = array();
    if (isset($_POST) && isset($_POST['value']))
        foreach ($_POST['value'] as $key => $value )
            if ($value != "") {
                if (preg_match('/(S[0-9]*)-./',$key,$a))
                    $keysubst = $a[1];
                else
                    $keysubst = $key;

                $options[] = array($_POST['option'][$keysubst] => $value);
            }

    // if we got files, extract bindata, and replace the array content
    if (isset($_FILES) && isset($_FILES['value']) && isset($_FILES['value']['tmp_name']))
        foreach ($_FILES['value']['tmp_name'] as $optiontag => $tmp_filename)
            if ($tmp_filename != "") { // there is an actual file
                // echo "<br/>There is a file...";
                if (preg_match('/(S[0-9]*)-./',$optiontag,$a)) {
                    $keysubst = $a[1];
                } else {
                    $keysubst = $key;
                }

                $blob = addslashes(downloadFile("file:///".$tmp_filename));

                $tentative = array($_POST['option'][$keysubst] => urlencode($blob));
                // print_r("Checking upload of type ".$_POST['option']['\"'.$keysubst.'\"']." for file $tmp_filename");
                if (check_upload_sanity($_POST['option'][$keysubst],$tmp_filename))
                    $options[] = $tentative;
            }

    // if we got eap:ca_url, convert it to eap:ca_file

    foreach ($options as $k => $iterate_option)
        foreach ($iterate_option as $name => $value) {
            if ($name == "eap:ca_url") {
                // echo "Downloading CA file ".$value;
                $pem = addslashes(X509::downloadCA($value));
                unset($options[$k]);
                $options[] = array("eap:ca_file" => urlencode($pem));
            }
        }

    // if we got general:logo_url, convert it to general:logo_file

    foreach ($options as $k => $iterate_option)
        foreach ($iterate_option as $name => $value) {
            if ($name == "general:logo_url") {
                // echo "Downloading logo from ".$value;
                $bindata = downloadFile($value);
                unset($options[$k]);
                if (check_upload_sanity("internal:logo_from_url",$bindata) )
                    $options[] = array("general:logo_file" => urlencode(addslashes($bindata)));
            }
        }

    // convert geo_lat and geo_long to geo_coordinate array

    /* echo "<pre>";
            print_r($_POST);
            echo "</pre>"; */
    if (       isset($_POST['geo_long'])
            && isset($_POST['geo_lat'])
            && $_POST['geo_long'] != ""
            && $_POST['geo_lat'] != "" )
        $options[] = array("general:geo_coordinates" => serialize(array("lon" => $_POST['geo_long'], "lat" => $_POST['geo_lat'])));



    // print_r($options);

    foreach ($options as $optionkey => $iterate_option)
        foreach ($iterate_option as $name => $value) {
            if (preg_match("/_file$/",$name)) {
                if (preg_match("/^ROWID-.*-([0-9]+)/",$value,$retval)) {
                    unset($killlist[$retval[1]]);
                    continue;
                } else {
                    $value = urldecode($value);
                }


            }
            if ($object instanceof IdP)
                $object->addAttribute($name, $value);
            elseif ($object instanceof Profile)
                $object->addAttribute($name, $value, 0);
        }

    // print_r($options);

    return $killlist;
}

function getBlobFromDB($ref) {

    $table = "";
    $rowindex = "";

    if (preg_match("/IdP/", $ref)) $table = "institution_option";
    if (preg_match("/Profile/", $ref)) $table = "profile_option";
    preg_match("/.*-([0-9]*)/", $ref, $rowindexmatch);
    $rowindex = $rowindexmatch[1];

    if ($table == "" || $rowindex == "") return;

    $blob = fetch_raw_db_data($table, $rowindex);
    if (!$blob) return FALSE;
    return $blob;
}

function display_size($number) {
    if ($number > 1024*1024)
        return round($number / 1024 / 1024, 2)." MiB";
    if ($number > 1024)
        return round($number / 1024, 2)." KiB";
    return $number." B";

}

function previewCAinHTML($ca_reference) {
    $found = preg_match("/^ROWID-.*/", $ca_reference);
    if (! $found )
        return "<div>"._("Error, ROWID expected.")."</div>";

    $ca_blob = getBlobFromDB($ca_reference);

    $func = new X509;
    $details = $func->processCertificate($ca_blob);
    if ($details === FALSE) return _("There was an error processing the certificate!");
    $details['name'] = preg_replace('/(.)\/(.)/', "$1<br/>$2", $details['name']);
    $details['name'] = preg_replace('/\//', "", $details['name']);
    return "<div class='ca-summary'>".$details['name']."</div>";
}

function previewImageinHTML($image_reference) {
    $found = preg_match("/^ROWID-.*/", $image_reference);
    if (! $found )
        return _("<div>Error, ROWID expected.</div>");
    return "<img width='200' src='inc/imagepreview.php?id=".$image_reference."' alt='"._("Preview of logo file")."'/>";
}

function previewInfoFileinHTML($file_reference) {
    $found = preg_match("/^ROWID-.*/", $file_reference);
    if (! $found )
        return _("<div>Error, ROWID expected.</div>");

    $file_blob = getBlobFromDB($file_reference);
    $fileinfo = new finfo();
    return "<div class='ca-summary'>"._("File exists")." (".$fileinfo->buffer($file_blob, FILEINFO_MIME_TYPE).", ".display_size(strlen($file_blob)).")<br/><a href='inc/filepreview.php?id=$file_reference'>"._("Preview")."</a></div>";
}


function infoblock($optionlist, $class, $level) {
    $retval = "";
    foreach ($optionlist as $option)
        if (preg_match('/^'.$class.'/', $option['name'] ) && $option['level'] == "$level") {
            $retval .= "
                             <tr>
                               <td>".display_name($option['name'])."</td>";
            if ($option['name'] == "general:logo_file") {
                $retval .= "
                               <td>".previewImageinHTML('ROWID-'.$option['level'].'-'.$option['row'])."</td>";
            } elseif ($option['name'] == "eap:ca_file") {
                $retval .= "
                               <td>".previewCAinHTML('ROWID-'.$option['level'].'-'.$option['row'])."</td>";
            } elseif ($option['name'] == "general:info_file") {
                $retval .= "
                               <td>".previewInfoFileinHTML('ROWID-'.$option['level'].'-'.$option['row'])."</td>";
            } elseif ($option['name'] == "general:geo_coordinates") {
                $coords = unserialize($option['value']);
                $retval .= "
                               <td><a href='http://maps.google.com/maps?q=".$coords['lat'].",".$coords['lon']."'>"._("Click to see location")."</a></td>";
            } else {
                $retval .= "
                               <td><strong>".$option['value']."</strong></td></tr>";
            };
        }

    return $retval;
}

function optiontext($defaultselect, $list, $prefill = 0) {

    $rowid = mt_rand();
    $retval = "<tr id='option-S$rowid' style='vertical-align:top'>
      <td>
         <select id='option-S$rowid-select' name='option[S$rowid]'>";

    $a = 0;

    foreach ($list as $key => $value) {
        $jsmagic = "onclick='
                               if (/_file$/.test(document.getElementById(\"option-S".$rowid."-select\").value)) {
                                  my_visibility(\"S$rowid-input-file\",\"file\");
                                  my_visibility(\"S$rowid-input-text\",\"file\")
                               } else {
                                  my_visibility(\"S$rowid-input-file\",\"text\");
                                  my_visibility(\"S$rowid-input-text\",\"text\")
                               }' ";
        $retval .="
             <option id='option-S".$rowid."-v-".$value."' value='".$value."' ".$jsmagic;
        if ($a == $defaultselect) {
            $retval .= "selected='selected'";
            if (preg_match("/_file$/",$value)) {
                $is_file = TRUE;
            } else {
                $is_file = FALSE;
            }
        }
        $retval .=">".display_name($value)."</option>";

        $a++;
    }


    $retval .="
          </select>
       </td>
       <td>";
    if (!$prefill) $retval .="
          <input type='text' style='display:".($is_file ? "none" : "block")."' name='value[S$rowid-1]' id='S".$rowid."-input-text'>
          <input type='file' style='display:".($is_file ? "block" : "none")."' name='value[S$rowid-2]' id='S".$rowid."-input-file' size='10'>";
    if ($prefill) {
        if (!$is_file) {
            if ($value == "general:geo_coordinates") {
                $coords = unserialize($prefill);
                $link = "<a href='http://maps.google.com/maps?q=".$coords['lat'].",".$coords['lon']."'>"._("Click to see location")."</a>";
                $retval    .="
          <input readonly style='display:none' type='text' name='value[S$rowid-1]' id='S".$rowid."-input-text' value='$prefill'>
                        $link";
            } else {
                $retval    .="
          <input readonly type='text' style='display:block' name='value[S$rowid-1]' id='S".$rowid."-input-text' value='$prefill'>";
            }
        }
        if ($is_file) {
            if ($value == "eap:ca_file" ) {
                $retval    .="
          <input readonly type='text' style='display:none' name='value[S$rowid-1]' id='S".$rowid."-input-text' value='".urlencode($prefill)."'>
          ".previewCAinHTML($prefill);
            }
            elseif ($value == "general:logo_file") {
                $retval    .="
          <input readonly type='text' style='display:none' name='value[S$rowid-1]' id='S".$rowid."-input-text' value='".urlencode($prefill)."'>
          ".previewImageinHTML($prefill);
            }
            elseif ($value == "general:info_file") {
                $retval    .="
          <input readonly type='text' style='display:none' name='value[S$rowid-1]' id='S".$rowid."-input-text' value='".urlencode($prefill)."'>
          ".previewInfoFileinHTML($prefill);
            }
            else {
                $retval    .="
          <input readonly type='text' style='display:none' name='value[S$rowid-1]' id='S".$rowid."-input-text' value='".urlencode($prefill)."'>
          "._("file content");
            }
        }
    }
    $retval .="
       </td>
       <td>
          <button type='button' class='delete' onclick='deleteOption(\"option-S".$rowid."\")'>-</button>
       </td>
    </tr>";
    return $retval;
}

?>