<?php
require_once(dirname(dirname(dirname(__FILE__)))."/config/_config.php");
include_once("Federation.php");
include_once("IdP.php");
include_once("Profile.php");
include_once("Helper.php");
include_once("inc/common.inc.php");

if (isset($_GET['lang'])) {
    $ourlocale = set_locale("web_admin",$_GET['lang']);
    $lang_string = "&lang=".$_GET['lang'];
}
else {
    $ourlocale = set_locale("web_admin");
    $lang_string="";
}

// let's check if the inst handle actually exists in the DB

$my_inst = new IdP($_GET['inst_id']);

if (! $my_inst instanceof IdP) {
    echo "<p>"._("The page was called with a non-existing IdP identifier. This is not supposed to happen!")."</p>";
    exit(1);
}

if (isset($_GET['profile_id']) ) { // oh! We should edit an existing profile, not create a new one!
    $inst_profiles = $my_inst->listProfiles();
    foreach ($inst_profiles as $profile)
        if ($profile->identifier == $_GET['profile_id']) {
            $my_profile = $profile;
            $wizard_style = FALSE;
            $edit_mode = TRUE;
            $prefill_name = $my_profile->name;
            $anon_realm = $my_profile->getAnonymousIDSupport();
            $realm = $my_profile->getRealm();
            $prefill_methods = $my_profile->getEapMethodsinOrderOfPreference();
            $profile_options = $my_profile->getAttributes();
            break;
        }
    if ( $my_profile === FALSE) { // no profile <-> inst match!
        echo "<p>"._("The page was called with a Profile identifier, but there was no such profile in this institution!")."</p>";
        exit(1);
    }
} else {
    $wizard_style = TRUE;
    $edit_mode = FALSE;
    $prefill_name = "";
    $anon_realm = "";
    $realm = "";
    $prefill_methods = array();
    $profile_options = array();
}

// maybe we were asked delete an existing profile? check for that...

if ($edit_mode && isset($_POST['profile_action']) && $_POST['profile_action'] == "delete") {
    $my_profile->destroy();
    header("Location: dashboard.php?inst_id=$my_inst->identifier$lang_string");
}

$idpoptions = $my_inst->getAttributes();

// did the user submit info? If so, submit to DB and go on to the 'dashboard' or 'next profile' page.

if ( isset( $_POST['submitbutton']) ) { // here we go

    if ($edit_mode) { // we need to write new data into an existing profile
        // i.e. flush/overwrite everything
        $profile = $my_profile;
        $profile->setName($_POST['profile_name']);
        $profile->flushSupportedEapMethods();
    }
    else {
        $profile = $my_inst->newProfile($_POST['profile_name']);
    }

    if (! $profile instanceof Profile) {
        echo _("Darn! Could not get a proper profile handle!");
        exit(1);
    };


    foreach (EAP::listKnownEAPTypes() as $a) {
        if (isset($_POST[display_name($a)]) && isset($_POST[display_name($a)."-priority"])) {
            $profile->addSupportedEapMethod($a, $_POST[display_name($a)."-priority"]);
        }
    }

    if (isset($_POST['realm']))
        $profile->setRealm($_POST['realm']);
    if (isset($_POST['anon_support']) && $_POST['anon_support'] == "on" && isset($_POST['realm'])) {
        $profile->setAnonymousIDSupport(true, $_POST['realm']);
    } else {
        $profile->setAnonymousIDSupport(false);
    };


    $remaining_attribs = $profile->beginflushAttributes(0);
    $killlist = processSubmittedFields($profile,$remaining_attribs);
    $profile->commitFlushAttributes($killlist);
    if ( $_POST['submitbutton'] == "submitted-dashboard" )
        header("Location: dashboard.php?inst_id=$my_inst->identifier$lang_string");
}
header("Content-Type:text/html;charset=utf-8");
?>
<!DOCTYPE html>
<html>
    <head>
        <meta http-equiv="Content-Type" content="text/html; charset=UTF-8" />
        <meta http-equiv="content-language" content="<?php echo $ourlocale;?>">
        <link rel="stylesheet" type="text/css" href="../resources/css/cat.css" />
        <script src="js/option_expand.js" type="text/javascript"></script>
        <title>
            <?php
            if ($wizard_style)
                echo _("eduroam CAT: IdP Enrollment Wizard (Step 3)");
            else
                printf(_("eduroam CAT: Edit Profile '%s' ..."),$prefill_name);
            ?>
        </title>
    </head>
    <body>
        <?php
        include "inc/admin_header.php";
        // If we got here, and have submitted POST data, then we apparently have just run successfully through
        // profile generation, and the user wants to add another one. Tell him so.
        if ( $_POST['submitbutton'] == "submitted-more" )
            echo "<p><font color='#7fff7f'>"._("Profile generated successfully!")."</font></p>";
        ?>
        <h1>
            <?php
            if ($wizard_style)
                echo _("Step 3: Defining a user group profile");
            else
                printf(_("Edit profile '%s' ..."), $prefill_name);
            ?>
        </h1>
        <div class='infobox'>
            <h2><?php echo _("General Institution Details");?></h2>
            <table>
                <tr>
                    <td>
                        <?php echo ""._("Country:")?>
                    </td>
                    <td>
                        <strong><?php echo Federation::name($my_inst->federation);?></strong>
                    </td>
                </tr>
                <tr>
                    <td>
                        <?php echo _("Institution name:");?>
                    </td>
                    <td>
                        <strong><?php echo $my_inst->name;?></strong>
                    </td>
                </tr>
                <?php echo infoblock($idpoptions, "general", "IdP");?>
            </table>
        </div>
        <div class="infobox">
            <h2><?php echo _("Global Helpdesk Details");?></h2>
            <table><?php echo infoblock($idpoptions, "support", "IdP");?></table>
        </div>
        <div class='infobox'>
            <h2><?php echo _("Global EAP Options");?></h2>
            <table><?php echo infoblock($idpoptions, "eap", "IdP");?></table>
        </div>
        <?php
        echo "<form enctype='multipart/form-data' action='profile_wizard.php?inst_id=$my_inst->identifier".($edit_mode ? "&profile_id=".$my_profile->identifier : "")."$lang_string' method='post'>
                <input type='hidden' name='MAX_FILE_SIZE' value='".Config::$MAX_UPLOAD_SIZE."'>";
        ?>
        <fieldset class="option_container">
            <legend>
                <strong><?php echo _("General Profile properties");?></strong>
            </legend>
            <?php
            if ($wizard_style)
                echo "<p>"._("We will now define a profile for your user group(s).  You can add as many profiles as you like by choosing the appropriate button on the end of the page. After we are done, the wizard is finished and you will be taken to the main IdP administration page.")."</p>";
            ?>
            <h3><?php echo _("Profile Name and RADIUS realm");?></h3>
            <?php
            if ($wizard_style)
                echo "<p>"._("First of all we need a name for the profile. This will be displayed to end users, so you may want to choose a descriptive name like 'Professors', 'Students of the Faculty of Bioscience', etc.</p>");
            ?>
            <label>
                <?php echo _("Profile Name:")."<input name='profile_name' title='Profile Name' value='$prefill_name'/>"; ?>
            </label>
            <br/>
            <?php
            if ($wizard_style)
                echo "<p>"._("You can tell us your RADIUS realm. This is useful if you want to use the sanity check module later, which tests reachability of your realm in the eduroam infrastructure. It is required to enter the realm name if you want to support anonymous outer identities (see below).")."</p>";
            ?>
            <label>
                <?php
                echo _("Realm").": <input id='realm' name='realm' value='$realm' onkeyup='
                                 if (this.value.length > 0)
                                      { this.form.elements[\"anon_support\"].removeAttribute(\"disabled\");
                                        document.getElementById(\"anon_support_label\").removeAttribute(\"style\");
                                      } else
                                      { this.form.elements[\"anon_support\"].checked = false;
                                        this.form.elements[\"anon_support\"].setAttribute(\"disabled\", \"disabled\");
                                        document.getElementById(\"anon_support_label\").setAttribute(\"style\", \"color:#999999\");
                                      };'/>";?>
            </label>
            <h3><?php echo _("Anonymity Support");?></h3>

            <?php
            if ($wizard_style)
                echo "<p>"._("Some installers support a feature called 'Anonymous outer identity'. If you don't know what this is, please read <a href='anonymous.html'>this article</a>. Do you want us to generate installers with anonymous outer identities where available? You need to fill out the 'Realm' field above for this to work.")."</p>";
            ?>
            <p>
                <label>
                    <?php
                    echo "<span id='anon_support_label' style='".($realm == "" ? "color:#999999" : "" )."'>"._("Enable Anonymous Outer Identity:")."</span>
                             <input type='checkbox' ".($anon_realm != "" ? "checked" : "" ).($realm == "" ? "disabled" : "" )." name='anon_support' />";
                    ?>
                </label>
            </p>


        </fieldset>
        <fieldset class="option_container">
            <legend><strong><?php echo _("Supported EAP types");?></strong></legend>
            <?php
            if ($wizard_style)
                echo "<p>"._("Now, we need to know which EAP types your IdP supports. If you support multiple EAP types, you can assign every type a priority (1=highest). This tool will always generate an automatic installer for the EAP type with the highest priority; only if the user's device can't use that EAP type, we will use an EAP type further down in the list.")."</p>";
            ?>
            <table>
                <?php
                function priority ($eap_type, $isenabled, $priority) {
                    echo "<td><select id='$eap_type-priority' name='$eap_type-priority' ".(!$isenabled ? "disabled='disabled'" : "").">";
                    for ($a=1; $a<7; $a=$a+1)
                        echo "<option id='$eap_type-$a' value='$a' ".( $isenabled && $a == $priority ? "selected" : "" ).">$a</option>";
                    echo "</select></td>";
                }

                function inherited_options ($idpwideoptions, $eap_type, $is_visible) {
                    echo "<td><div style='".(!$is_visible ? "visibility:hidden" : "")."' class='inheritedoptions' id='$eap_type-inherited-global'>";

                    $eapoptions = array();

                    foreach ($idpwideoptions as $option)
                        if ( $option['level'] == "IdP" && preg_match('/^eap/', $option['name']) )
                            $eapoptions[] = $option['name'];

                    $eapoptions = array_count_values($eapoptions);

                    if (count ( $eapoptions ) > 0 ) {
                        echo "<strong>"._("EAP options inherited from Global level:")."</strong><br />";
                        foreach ($eapoptions as $optionname => $count)
                            printf(_("(%dx) %s")."<br />",$count,display_name($optionname));
                    }

                    echo "</div></td>";
                }

                $methods = EAP::listKnownEAPTypes();
                foreach ($methods as $a) {
                    $display = display_name($a);
                    $enabled = FALSE;
                    if ($edit_mode)
                        foreach ($prefill_methods as $prio => $value) {
                            if  (display_name($a) == display_name($value)) {
                                echo "found ".display_name($a)." as ".$prio;
                                $enabled = TRUE;
                                $countactive=$prio+1;
                            }
                        }


                    echo "
                <tr style='vertical-align:top'>
                    <td>
                        <label>
                      <input type='checkbox'
                                name='$display' ".( $enabled ? "checked" : "" )."
                                value='$display'
                                onclick='if (this.checked) {
                                            this.form.elements[\"$display-priority\"].removeAttribute(\"disabled\");
                                            document.getElementById(\"$display-inherited-global\").setAttribute(\"style\", \"visibility:visible\");
                                         } else {
                                            this.form.elements[\"$display-priority\"].setAttribute(\"disabled\", \"disabled\");
                                            document.getElementById(\"$display-inherited-global\").setAttribute(\"style\", \"visibility:hidden\"); }' />$display
                        </label>
                    </td>";

                    priority($display, $enabled, $countactive);
                    inherited_options($profile_options, $display, $enabled);
                }

                echo "</tr>";
                ?>
            </table>
        </fieldset>
        <fieldset class="option_container" id="helpdesk_override">
            <legend><strong><?php echo _("Helpdesk Details for this profile");?></strong></legend>
            <p>
                <?php echo _("If you specify an option here, it will override the global option(s) with the same name, if any.");?>
            </p>
            <table id="expandable_support_options">
                <?php
                $class="support";
                if ($edit_mode) {
                    $prepopulate=array();
                    $existing_attribs=$my_profile->getAttributes();
                    foreach ($existing_attribs as $existing_attribute)
                        if ( $existing_attribute['level'] == "Profile" ) $prepopulate[] = $existing_attribute;
                }
                include "inc/add_option.php";
                ?>
            </table>
            <button type='button' class='newoption' onclick='addDefaultSupportOptions("<?php echo $lang_string;?>")'><?php echo _("Add new option"); ?></button>
        </fieldset>
        <fieldset class="option_container" id="eap_override">
            <legend><strong><?php echo _("EAP Details for this profile");?></strong></legend>
            <p>
                <?php echo _("If you specify an option here, it will override the global option(s) with the same name, if any.");?>
            </p>
            <table id="expandable_eapserver_options">
                <?php
                $class="eap";
                include "inc/add_option.php";
                ?>
            </table>
            <button type='button' class='newoption' onclick='addDefaultEapServerOptions("<?php echo $lang_string;?>")'><?php echo _("Add new option"); ?></button>
        </fieldset>
        <?php
        if ($wizard_style)
            echo "<p>"._("When you are sure that everything is correct, please click on
                 <button type='submit' name='submitbutton' value='submitted-dashboard'>Continue to dashboard</button>")."</p>
                 <p>"._("If you want to set up another profile after this one, please click on
                 <button type='submit' name='submitbutton' value='submitted-more'>Add another profile</button>")."</p></form>";
        else
            echo "<p><button type='submit' name='submitbutton' value='submitted-dashboard'>"._("Save data")."</button></p></form>
                 <form action='dashboard.php?inst_id=$my_inst->identifier$lang_string' method='post'><button type='submit' name='abortbutton' value='aborted'>"._("Discard changes")."</button></form>";

        include "inc/admin_footer.php";
        ?>
    </body>
</html>
