<?php
/**
 * This file contains the Federation class.
 * 
 * @author Stefan Winter <stefan.winter@restena.lu>
 * @author Tomasz Wolniewicz <twoln@umk.pl>
 * 
 * @package Developer
 * 
 */

require_once('IdP.php');

/**
 * This class represents an eduroam federation.
 * It is semantically a country(!). Do not confuse this with a TLD; a federation
 * may span more than one TLD, and a TLD may be distributed across multiple federations.
 *
 * Example: a federation "fr" => "France" may also contain other TLDs which
 *              belong to France in spite of their different TLD
 * Example 2: Domains ending in .edu are present in multiple different
 *              federations
 *
 * @author Stefan Winter <stefan.winter@restena.lu>
 * @author Tomasz Wolniewicz <twoln@umk.pl>
 *
 * @license http://ticker.eduroam.lu/cat/LICENSE
 *
 * @package Developer
 */
class Federation {

    /**
     *
     * Constructs a Federation object.
     *
     * @param string $fedname - textual representation of the Federation object
     *        Example: "lu" (for Luxembourg)
     */

    public function __construct($fedname) {
 /* Federations are created in DB with bootstrapFederation, and listed via listFederations
 */
        $this->identifier = $fedname;
    }
/**
 * Prepares a list of countries known to the CAT
 * @param int $active_only is set and nonzero will cause that only countries with some institutions underneath will be listed
 * @return array Array indexed by (uppercase) lang codes and sorted according to the current locale
 */
public function printCountryList($active_only = 0) {
$olddomain = Globals::$thedomain;
set_locale("core");
   $Country_list = array(
      'CZ'=> _("Czech"),
      'DK'=> _("Denmark"),
      'FI'=> _("Finland"),
      'FR'=> _("France"),
      'DE'=> _("Germany"),
      'GR'=> _("Greece"),
      'HR'=> _("Croatia"),
      'IE'=> _("Ireland"),
      'IT'=> _("Italy"),
      'JP'=> _("Japan"),
      'HU'=> _("Hungary"),
      'LU'=> _("Luxembourg"),
      'NL'=> _("Netherlands"),
      'NO'=> _("Norway"),
      'PL'=> _("Poland"),
      'PT'=> _("Portugal"),
      'SI'=> _("Slovenia"),
      'ES'=> _("Spain"),
      'SE'=> _("Sweden"),
      'CH'=> _("Switzerland"),
      'TR'=> _("Turkey"),
      'US'=> _("USA"),
      'GB'=> _("UK"),
   );

   if($active_only) {
     $FED = Federation::listFederations();
     foreach ($FED as $F) {
       $c = strtoupper($F->identifier);
       $C[$c] = isset($Country_list[$c]) ? $Country_list[$c] : $c;
     }
   } else
    $C = $Country_list;
   asort($C, SORT_LOCALE_STRING );
   set_locale($olddomain);
   return($C);
}
   /**
     *
     * @var array List of supported countries and their English names
     */
  private static $code_list = array(
  'lu' => 'Luxembourg',
  'pl' => 'Poland',
  'de' => 'Germany',
  'test'=>'TEST Country',
  'es' => 'Spain',
  'nl' => 'The Netherlands',
  'gb' => 'Great Britain',
  );


  /**
   *
   * @param string $code The shortname of the country
   * @return string The display name of the country
   */
  public static function name($code) {
    if(isset( Federation::$code_list[$code]))
    return Federation::$code_list[$code];
    else
    return $code;
  }
  /**
   *
   * @return array The list of countries; consisting of an array where key = shortname, value = display name
   */
  public static function country_list () {
      return Federation::$code_list;
  }

    /**
     * Lists all federations that exist in the database
     *
     * @return array (Array of Federation instances)
     */

  public static function listFederations() {

        DBConnection::handle();

        $q = "SELECT DISTINCT LOWER(country) AS country
              FROM institution ORDER BY country";
        $federations = mysql_query($q);

        $returnarray = array();
        while ( $a = mysql_fetch_object($federations)->country )
                $returnarray[] = new Federation($a);

        return $returnarray;
    }

    /**
     * Lists all Identity Providers in this federation
     *
     * @return array (Array of IdP instances)
     *
     */

    public static function bootstrapFederation($fedname, $firstinst) {

        if (array_key_exists($fedname, Federation::country_list())) {
                $newfed = new Federation($fedname);
                $newinst = $newfed->newIdP($firstinst);
                return $newinst;
        } else
            return FALSE;
    }

    public function newIdP($name) {
        $q_newidp = "INSERT INTO institution (inst_name, country) VALUES('$name','$this->identifier')";
        $newidp = mysql_query($q_newidp);
        $q_identifier = "SELECT inst_id FROM institution WHERE inst_name = '".$name."' AND country = '$this->identifier'";
        $identifier_result = mysql_query($q_identifier);
        $id = mysql_fetch_object($identifier_result)->inst_id;
        return $id;
    }
    /**
     * Textual short-hand representation of this Federation
     *
     * @var string (Example: "fr")
     *
     */

    public $identifier;

    /**
     * Lists all Identity Providers in this federation
     *
     * @param int $active_only if and set to non-zero will
     * cause listing of only those institutions which have some profiles defined.
     * @return array (Array of IdP instances)
     *
     */
  public function listIdentityProviders($active_only = 0) {

        DBConnection::handle();
        if($active_only) {
        $q  = "SELECT distinct institution.inst_id AS inst_id,
               institution.inst_name AS inst_name FROM institution
               JOIN profile ON institution.inst_id = profile.inst_id
               WHERE institution.country = '$this->identifier' 
               ORDER BY inst_name";
        } else {
        $q  = "SELECT inst_id,inst_name FROM institution
               WHERE country = '$this->identifier' ORDER BY inst_name";
        }
        debug(5,$q."\n");
        $allIDPs = mysql_query($q);
        $returnarray = array();
        while ($a = mysql_fetch_object($allIDPs)) {
            $idp = new IdP($a->inst_id);
            $A = array ('entityID'=>$idp->identifier,
                 'title'=>$idp->name, 'country'=>strtoupper($idp->federation));
            $returnarray[] = $A;
        }
        return $returnarray;
    }

    public static function listAllIdentityProviders() {
        DBConnection::handle();
//        $q  = "SELECT inst_id, inst_name FROM institution";
//        $q .= " ORDER BY inst_name";
        $q  = "SELECT distinct institution.inst_id AS inst_id, institution.inst_name AS inst_name FROM institution";
        $q .= " JOIN profile ON institution.inst_id = profile.inst_id ORDER BY inst_name"; debug(5,$q."\n");
        $allIDPs = mysql_query($q);
        $returnarray = array();
        while ($a = mysql_fetch_object($allIDPs)) {
            $idp = new IdP($a->inst_id);
            $A = array ('entityID'=>$idp->identifier,
                 'title'=>$idp->name, 'country'=>strtoupper($idp->federation));
            $at = $idp->getAttributes('general:geo_coordinates');
             if($at) {
               if(count($at) > 1) {
                 $at1=array();
                 foreach($at as $a)
                   $at1[] = unserialize($a['value']);
               } else
                 $at1=unserialize($at[0]['value']);
              $A['geo'] = $at1;
            }
            $returnarray[] = $A;
        }
        return $returnarray;
}
}

?>
