<?php
/**
 * This file creates MS Windows XP installers
 * It supports TTLS only (so far, but with hooks open for others)
 *
 * @author Tomasz Wolniewicz <twoln@umk.pl>
 *
 * @package ModuleWriting
 */

require_once('Device.php');

/**
 * 
 * @author Tomasz Wolniewicz <twoln@umk.pl>
 * @package ModuleWriting
 */
class Device_WindowsXP extends DeviceConfig {
    final public function __construct() {
      $this->supportedEapMethods  = array(EAP::$TTLS_PAP);
//      $this->supportedEapMethods  = array(EAP::$TLS, EAP::$PEAP_MSCHAP2, EAP::$TTLS_PAP);
      debug(4,"This device supports the following EAP methods: ");
      debug(4,$this->supportedEapMethods);
    }

  public function writeInstaller() {
      $dom = textdomain(NULL);
      textdomain("devices");
   // create certificate files and save their names in $CA_files arrary
     $CA_files = $this->saveCertificateFiles('der');

     $SSIDs = $this->attributes['internal:SSID'];


     if ($this->selected_eap == EAP::$TLS || $this->selected_eap == EAP::$PEAP_MSCHAP2) {
       $WindowsProfile = array();
       $i = 0;
       foreach ($SSIDs as $ssid => $enc) {
          $WindowsProfile[$i] = $this->writeWLANprofile ($ssid,$ssid,'WPA2','AES',$this->attributes,$i);
          $i++;
       }
     } elseif($this->selected_eap == EAP::$TTLS_PAP) {
       $WindowsProfile = $this->writeSW2profile($this->attributes,$CA_files);
     } else {
       error("  this EAP type is not handled yet");
       return;
     }
    debug(4,"WindowsProfile"); debug(4,$WindowsProfile);
    
    $this->writeProfilesNSH($WindowsProfile, $CA_files);
    $this->copyFiles($this->selected_eap);
    if(isset($this->attributes['internal:logo_file']))
       $this->combineLogo($this->attributes['internal:logo_file']);
    $this->writeMainNSH($this->selected_eap,$this->attributes);
    $this->compileNSIS();
    $installer_path = $this->signInstaller($this->attributes); 

    textdomain($dom);
    return($installer_path);  
  }

// $servers is an array of allowed server names (regular expressions allowed)
// $ca is an array of allowed CA fingerprints
//   writeWLANprofile ('eduroam','eduroam','WPA2','AES',$eap,$servers,$ca,$i)

/**
 * produce PEAP and TLS configuration files for Windows XP
 */
  private function writeWLANprofile($wlan_profile_name,$ssid,$auth,$encryption,$attr,$i) {
    $eap = $this->selected_eap;
    if ($eap != EAP::$TLS && $eap != EAP::$PEAP_MSCHAP2) {
      debug(2,"this method only allows TLS or PEAP");
      error("this method only allows TLS or PEAP");
     return;
    }
   $use_anon = $attr['internal:use_anon_outer'] [0];
   $servers = implode(';',$attr['eap:server_name']);
   $ca_array = $attr['internal:CAs'][0];

debug(2,"Installer has been written into directory $this->FPATH\n");
debug(4,"WLAN_Profile:$wlan_profile_name:$encryption\n");
return("\"$wlan_profile_name\" \"$encryption\"");
}

private function glueServerNames($server_list) {
//print_r($server_list);
 $A0 =  array_reverse(explode('.',array_shift($server_list)));
 $B = $A0;
 if($server_list) {
   foreach($server_list as $a) {
   $A= array_reverse(explode('.',$a));
   $B = array_intersect_assoc($A0,$A);
   $A0 = $B;
   }
  }
  return(implode('.',array_reverse($B)));
}

/**
 * produce SecureW2 configuration for Windows XP
 */

  private function writeSW2profile($attr,$ca_path_array) {
// global section
   $ca_array = $attr['internal:CAs'][0];
   $sw2_server = $this->glueServerNames($attr['eap:server_name']);
   $SSIDs = $attr['internal:SSID'];

$P = array();

$profile_file_contents = '[Version]
Signature = "$Windows NT$"
Provider = "SecureW2"
Config = 7

[WZCSVC]
Enable = AUTO
Restart = TRUE

[DOT3SVC]
Enable = AUTO
Restart = TRUE

[Certificates]
';
$i = 1;
if($ca_path_array)
foreach ($ca_path_array as $CA) {
  $profile_file_contents .= "Certificate.$i = $CA\n";
     $i++;
}
// profiles section
$j = 1;
debug(4,"SW2-HERE\n");
foreach ($SSIDs as $ssid => $enc) {
$profile_file_contents .= '
[SSID.'.$j.']
Name = "'.$ssid.'"
SSID = "'.$ssid.'"
Profile = "DEFAULT"
AuthenticationMode = "WPA2"
EncryptionType = "AES"
';
$j++;
$P[] = '"'.$ssid.'" "AES"';
}

// user section
$profile_file_contents .= '
[Profile.1]
Name = "DEFAULT"
Description = "'._("Login credentials").':"
UserName = PROMPTUSER
PromptUserForCredentials = FALSE
UseAnonymousOuterIdentity = TRUE
UseEmptyOuterIdentity = TRUE
VerifyServerName = TRUE
ServerName = "'.$sw2_server.'"
VerifyServerCertificate = TRUE
';
$i = 0;
foreach ($ca_array as $CA) {
     $profile_file_contents .= "TrustedRootCA.$i = ".$CA['sha1']."\n";
     $i++;
}


$f_name = "SecureW2.INF";
$sw2_f = fopen($f_name,'w');

 fwrite($sw2_f,$profile_file_contents);
fclose($sw2_f);
debug(2,"Installer has been written into directory $this->FPATH\n");
return($P);
}


private function writeMainNSH($eap,$attr) {
debug(4,"writeMainNSH"); debug(4,$attr);
$fcontents = '';
$LANGS=array('pl'=>'Polish','en'=>'English','de'=>'German');

$lang = isset($LANGS[$this->lang_index]) ? $LANGS[$this->lang_index] : "English";

// $fcontents = "!define ALLOW_VISTA\n";
if($eap["OUTER"] == PEAP) 
  $eap_str = 'PEAP';
if($eap["OUTER"] == TLS) 
  $eap_str = 'TLS';
if($eap["OUTER"] == TTLS) 
  $eap_str = 'TTLS';

$fcontents .= '!define '.$eap_str;
if($attr['internal:profile_count'][0] > 1)
$fcontents .= "\n".'!define USER_GROUP "'.$attr['internal:profile_name'][0].'"';
$fcontents .= '
!define APPLICATION "eduroam installer for '.$attr['internal:inst_name'][0].'"
!define VERSION "1.00"
!define INSTALLER_NAME "eduroam_xp.exe"
!define LANG "'.$lang.'"
';
if(isset($attr['general:info_file'])) { 
  debug(4,"Info file: ".$attr['general:info_file']['mime']."\n");
  if ($attr['internal:info_file'][0]['mime'] == 'rtf' || $attr['internal:info_file'][0]['mime'] == 'txt') 
     $fcontents .= '!define LICENSE_FILE "'.$attr['internal:info_file'][0]['name']."\"\n";
  else
     $fcontents .= '!define EXTERNAL_INFO "'.$attr['internal:info_file'][0]['name']."\"\n";
}
  

$fcontents .= ';--------------------------------
!define ORGANISATION "'.$attr['internal:inst_name'][0].'"
!define SUPPORT "'.$attr['support:email'][0].'"
!define URL "'.$attr['support:url'][0].'"

!ifdef TLS
';
//TODO this must be changed with a new option
$fcontents .= '!define TLS_CERT_STRING "certyfikaty.umk.pl"
!define TLS_FILE_NAME "cert*.p12"
!endif
';

$f = fopen('main.nsh','w');
fwrite($f, $fcontents);
fclose($f);

}

private function writeProfilesNSH($P,$CA) {
debug(4,"writeProfilesNSH");
debug(4,$P);
$fcontents = '';
  foreach($P as $p) 
    $fcontents .= "!insertmacro define_wlan_profile $p\n";
$f = fopen('profiles.nsh','w');
fwrite($f, $fcontents);
fclose($f);

$fcontents = '';
  if($CA)
    foreach ($CA as $c)
      $fcontents .= "!insertmacro install_ca_cert \"$c\"\n";
$f = fopen('certs.nsh','w');
fwrite($f, $fcontents);
fclose($f);
}

private function copyFiles ($eap) {
debug(4,"copyFiles start\n");
   $result;
   $result = $this->copyFile('cat_bg.bmp');
   $result = $result && $this->copyFile('eduroam32.ico');
   $result = $result && $this->copyFile('eduroam_150.bmp');
   $this->translateFile('common.inc','common.nsh',1);
   if($eap["OUTER"] == TTLS)  {
     $this->translateFile('ttls.inc','eduroam.NSI',1);
     $result = $result && $this->copyFile('SecureW2_EAP_Suite_113.zip');
    } else {
     $this->translateFile('peap_tls.inc','eduroam.NSI',1);
     $result = 1;
    }
debug(4,"copyFiles end\n");
   return($result);
}

private function combineLogo($Logos) {
// maximum size to which we want to resize
  $max_size= 120;
// logo wull be shited up by this much
  $vshift = 20;
  $bg_image = new Imagick('cat_bg.bmp');
  $bg_image_size = $bg_image->getImageGeometry();
  $logo = new Imagick($Logos[0]['name']);
  $logo_size = $logo->getImageGeometry();
  $max = max($logo_size);
  debug(4,"Logo size: "); debug(4,$logo_size); debug(4,"max=$max\n");
// resize logo if necessary
  if($max > $max_size) {
    if($max == $logo_size['width'])
       $logo->scaleImage($max_size,0);
    else
       $logo->scaleImage(0,$max_size);
  }
  $logo_size = $logo->getImageGeometry();
  debug(4,"New logo size: "); debug(4,$logo_size); 
// calculate logo offsets for composition with the background
  $hoffset = round(($bg_image_size['width'] - $logo_size['width'])/2); 
  $voffset = round(($bg_image_size['height'] - $logo_size['height'])/2) - $vshift; 
     
//logo image is put on top of the background
$bg_image->compositeImage($logo, $logo->getImageCompose(), $hoffset, $voffset);

//new image is saved as the background
$bg_image->writeImage('cat_bg.bmp');
}

private function compileNSIS() {
  $o = system('makensis -V0 eduroam.NSI');
}

private function signInstaller($attr) {
   
   $e = 'eduroam_xp_'.preg_replace('/ +/','_',$attr['internal:inst_name'][0]).'.exe';
   if($this->sign)
      $o = system($this->sign." eduroam_xp.exe $e > /dev/null");
   else
      rename("eduroam_xp.exe",$e);
   return $e;
}

/*
  we will be writing the following wlan_profiles:
   1. name=>eduroam, ssid=>eduroam, auth=>WPA2, encryption=>AES
   2. name=>(tkip_profile_name), ssid=>eduroam, auth=>WPA, encryption=>TKIP
   optionally
   3. name=>eduroam-local, ssid=>(additional_SSID), auth=>WPA2, encryption=>AES
   4. name=>(tkip_profile_name)-local, ssid=>(additional_SSID), auth=>WPA, encryption=>TKIP
   
*/
}

?>
