<?php
require_once(dirname(dirname(dirname(__FILE__)))."/config/_config.php");
include_once("Helper.php");
include_once("IdP.php");
include_once("Profile.php");
include_once("DeviceFactory.php");
include_once("devices/devices.php");
include_once("inc/common.inc.php");

if (isset($_GET['lang'])) {
    $ourlocale = set_locale("web_admin",$_GET['lang']);
    $lang_string = "&amp;lang=".$_GET['lang'];
}
else {
    $ourlocale = set_locale("web_admin");
    $lang_string="";
}
header("Content-Type:text/html;charset=utf-8");
?>
<!DOCTYPE html>
<html lang="<?php echo $ourlocale;?>">
    <head>
        <meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
        <title><?php echo _("Device Compatibility matrix");?></title>
        <link rel='stylesheet' type='text/css' href='../resources/css/cat.css' />
        <script src="js/popup_redirect.js" type="text/javascript"></script>
    </head>
    <body>
        <?php
        include "inc/admin_header.php";
        $my_inst = new IdP($_GET['inst_id']);

        if (! $my_inst instanceof IdP) {
            echo "<p>"._("The page was called with a non-existing IdP identifier. This is not supposed to happen!")."</p>";
            exit(1);
        }

        $my_profile = new Profile($_GET['profile_id']);
        if (! $my_profile instanceof Profile) {
            echo "<p>"._("The page was called with a Profile identifier, but there was no such profile in this institution!")."</p>";
            exit(1);
        }
        $preflist = $my_profile->getEapMethodsinOrderOfPreference();
        ?>
        <h1><?php printf(_("Device compatiblity matrix for %s of %s "),$my_profile->name,$my_inst->name);?></h1>
        <table class="compatmatrix">
            <tr>
                <th><?php echo _("Device");?></th>

                <?php
                foreach ($preflist as $method) {
                    echo "<th>".display_name($method)."</th>";
                }
                ?>

            </tr>
            <?php
            // okay, input is valid. Now create a table: columns are the EAP types supported in the profile,
            // rows are known devices

            $devices = Devices::listDevices();

            foreach ($devices as $index => $description) {
                echo "<tr>";
                echo "<td>".$description['display']."<br/><form><button disabled>"._("Redirect device for all EAP types")."</button></form></td>";
                $factory = new DeviceFactory($index);
                $defaultisset = FALSE;
                foreach ($preflist as $method) {
                    $encoded_method = serialize($method);
                    $slashed_method = addslashes($encoded_method);
                    $redirectform = "<div style='float:right'><form method='post' action='inc/toggleRedirect.inc.php?inst_id=$my_inst->identifier&amp;profile_id=$my_profile->identifier$lang_string' onsubmit='popupRedirectWindow(this); return false;'>
                                 <input type='hidden' name='eaptype' value='".(get_magic_quotes_gpc() == 0 ? $slashed_method : $encoded_method)."'>
                                 <input type='hidden' name='device' value='$index'>
                                 <button class='redirect' type='submit'>"._("R")."</button>
                              </form></div>";

                    $downloadform = "<form action='inc/installerDownload.inc.php?profile_id=$my_profile->identifier$lang_string' method='post'>
                                       <input type='hidden' name='device' value='$index'/>
                                       <button class='download'>"._("Download")."</button>
                                     </form>";
                    // first of all: if redirected, indicate by color

                    if ($my_profile->isBlacklisted($method, $description['module'])) {
                        echo "<td class='compat_redirected'>$redirectform";
                        if ($my_profile->isEapTypeDefinitionComplete($method) && ($method === $preflist[0] || $defaultisset == FALSE)) {
                                echo "$downloadform";
                                $defaultisset = TRUE;
                        }
                        echo "</td>";
                    } else
                    if (in_array($method, $factory->device->supportedEapMethods)) {
                        if (!$my_profile->isEapTypeDefinitionComplete($method)) {
                            echo "<td class='compat_incomplete'>$redirectform</td>";
                        }
                        elseif ($method === $preflist[0] || $defaultisset == FALSE) {
                            echo "<td class='compat_default'>$redirectform$downloadform</td>";
                            $defaultisset = TRUE;
                        }
                        else {
                            echo "<td class='compat_secondary'>$redirectform</td>";
                        }

                    }
                    else
                        echo "<td class='compat_unsupported'>$redirectform</td>";
                }
                echo "</tr>";
            }
            ?>
            <tr>
                <td></td>

                <?php
                foreach ($preflist as $method) {
                    echo "<td style='border-top: 2px solid #000000;'><form><button disabled>"._("Redirect EAP type for all devices")."</button></form></td>";
                }
                ?>

            </tr>
        </table>
        <p><strong>Legend:</strong></p>
        <table class="compatmatrix">
            <tr><td class="compat_redirected">&nbsp;&nbsp;&nbsp;</td> <td><?php echo _("redirection is set");?></td></tr>
            <tr><td class="compat_default">&nbsp;&nbsp;&nbsp;</td>    <td><?php echo _("will be offered on download site");?></td></tr>
            <tr><td class="compat_secondary">&nbsp;&nbsp;&nbsp;</td>  <td><?php echo _("configured, but not preferred EAP type");?></td></tr>
            <tr><td class="compat_incomplete">&nbsp;&nbsp;&nbsp;</td> <td><?php echo _("incomplete configuration");?></td></tr>
            <tr><td class="compat_unsupported">&nbsp;&nbsp;&nbsp;</td><td><?php echo _("not supported by CAT");?></td></tr>
        </table>
        <?php
        $buttontext = _("Return to dashboard");
        echo "<form method='post' action='dashboard.php?inst_id=$my_inst->identifier$lang_string'><button type='submit' name='submitbutton' value='return'>".$buttontext."</button></form>";?>
    </body>
</html>
