<?php

require_once(dirname(dirname(dirname(__FILE__)))."/config/_config.php");
include_once("Federation.php");
include_once("IdP.php");
include_once("Helper.php");

if (isset($_GET['lang'])) {
    $ourlocale = set_locale("web_admin",$_GET['lang']);
    $lang_string_init = "?lang=".$_GET['lang'];
    $lang_string = "&lang=".$_GET['lang'];
}
else {
    $ourlocale = set_locale("web_admin");
    $lang_string_init = "";
    $lang_string="";
}

// If we have POST data with a federation and IdP name, see if there is
// already an IdP listed. If not create one, and send user to the wizard
// page.

if (isset ($_POST['country']) and isset($_POST['instname'])  ) {
    if (! array_key_exists($_POST['country'], Federation::country_list()) ) { // oh-oh. The POSTed value is wrong!
        printf(_("<p>We know nothing about the country you have selected. Adding a country is a manual action.</p>
                <p>Please contact the %s operators to have this country added to the list.</p>"),Config::$PRODUCT_NAME);
        exit(1);
    };

    foreach (Federation::listFederations() as $fed_list )
        if ( $fed_list->identifier == $_POST['country']) $my_fed = $fed_list;

    if ( ! $my_fed instanceof Federation ) { // first inst in DB for this country. Create a federation and the IdP.
        $inst_id = Federation::bootstrapFederation($_POST['country'], $_POST['instname']);
        if (!$inst_id) exit(1);
        header("Location: idp_wizard.php?inst_id=$inst_id&wizard=true");
    } else {
        foreach ($my_fed->listIdentityProviders() as $key => $value )
            if ($value['title'] == $_POST['instname']) $my_inst = new IdP($value['entityID']);
        if ( ! $my_inst instanceof IdP ) { // new entry.
            $inst_id = $my_fed->newIdP($_POST['instname']);
            header("Location: idp_wizard.php?inst_id=$inst_id&wizard=true$lang_string");
        } else { // existing IdP, just redirect to status page
            header("Location: dashboard.php?inst_id=$my_inst->identifier$lang_string");
        };
    }

};
header("Content-Type:text/html;charset=utf-8");
?>
<!DOCTYPE html>
<html>
    <head>
        <meta http-equiv="Content-Type" content="text/html; charset=UTF-8">
        <meta http-equiv="content-language" content="<?php echo $ourlocale;?>">
        <link rel="stylesheet" type="text/css" href="../resources/css/cat.css" />
        <title><?php printf(_("%s: IdP enrollment wizard (step 1)"),Config::$PRODUCT_NAME); ?></title>
    </head>
    <body><?php include "inc/admin_header.php"; ?>
        <h1>
            <?php echo _("Step 1: Defining your Institution");?>
        </h1>
        <p>
            <?php printf(_("Welcome to the %s. This wizard will help you to enter all data about your IdP.
                          When the process is finished, you and your users will be able to download custom-made
                          installers for various platforms."),Config::$PRODUCT_NAME);?>
        </p>
        <h2>
            <?php echo _("Institution details");?>
        </h2>
        <form action='enrollment.php<?php echo $lang_string_init;?>' method='post'>
            <table>
                <tr>
                    <td style='width:400px'><?php echo _("My institution resides in the following country:")."
                        <br/>
                        <span class='tooltip'
                              onclick='alert(\""._("The list of countries is manually administered, because we need to map pretty-print names of countries to their ISO codes. If your country is missing, please contact OT. I am almost sure we figure out a more automatic way of doing this later.")."\")'>"._("My country is missing!")."
                        </span>";?>
                    </td>
                    <td>
                        <select name='country' id='country'>
                            <option value='(null)' selected='selected'>
                                <?php echo _("Please select from list...");?>
                            </option>
                            <?php
                            foreach (Federation::country_list() as $short => $name)
                                echo "<option value='$short'>$name</option>";
                            ?>
                        </select>
                    </td>
                </tr>
                <tr>
                    <td style='width:400px'><?php echo _("The name of my Institution is:"); ?></td>
                    <td><input name="instname" /></td>
                </tr>
                <tr>
                    <td>
                        <strong>
                            <?php echo _("Please review your selection thoroughly. Your institution will from now on be identified with these inputs."); ?>
                        </strong>
                    </td>
                    <td>
                    </td>
                </tr>
                <tr>

                    <?php echo _("<td>When you are sure that everything is correct, please click on </td><td><button type='submit' name='submitbutton' value='create'>Continue...</button></td>");?>
                </tr>
            </table>
        </form>
        <?php include "inc/admin_footer.php"; ?>
    </body>
</html>
